<?php
/************************************************************************
 * OVIDENTIA http://www.ovidentia.org                                   *
 ************************************************************************
 * Copyright (c) 2003 by CANTICO ( http://www.cantico.fr )              *
 *                                                                      *
 * This file is part of Ovidentia.                                      *
 *                                                                      *
 * Ovidentia is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2, or (at your option)  *
 * any later version.													*
 *																		*
 * This program is distributed in the hope that it will be useful, but  *
 * WITHOUT ANY WARRANTY; without even the implied warranty of			*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.					*
 * See the  GNU General Public License for more details.				*
 *																		*
 * You should have received a copy of the GNU General Public License	*
 * along with this program; if not, write to the Free Software			*
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,*
 * USA.																	*
************************************************************************/
require_once "base.php";

bab_functionality::includeFile('Ovml/Function');



/**
 * Create a thumbnail image
 * 
 * <OFThumbnail 
 * 		( id_directory_entry="" | id_user="" | id_article="" | path="absolute/path/to/image" ) 
 * 		[maxwidth="400"] 
 * 		[maxheight="400"] 
 * 		[resizemode="KEEP_ASPECT_RATIO | CROP_CENTER"] 
 * 		[default="path/to/default/image.jpg"]
 * 		[innerborderwidth="0"]
 * 		[innerbordercolor="#ffffff"]
 * 		[innerborder="0,#ffffff"]
 * 		[outerborderwidth="0"]
 * 		[outerbordercolor="#ffffff"]
 * 		[outerborder="0,#ffffff"]
 * 		[saveas=""]>
 *
 */
class Func_Ovml_Function_Thumbnail extends Func_Ovml_Function
{
	public function toString()
	{
		$T = bab_functionality::get('Thumbnailer');
		/*@var $T Func_Thumbnailer */
		
		
		if (!$T || !count($this->args))
		{
			return '';
		}
		
		
		$maxwidth = 400;
		$maxheight = 400;
		$innerbordercolor = '#ffffff';
		$outerbordercolor = '#cccccc';
		$innerborderwidth = 0;
		$outerborderwidth = 0;
		$resizemode = Func_Thumbnailer::KEEP_ASPECT_RATIO;
		$default = null;
		$saveas = null;
	
		
		foreach ($this->args as $p => $v) {
			switch (mb_strtolower(trim($p))) {
				
				case 'maxwidth':
					
					if ($v <= 0)
					{
						trigger_error(sprintf('Invalid parameter maxwidth=%s in OFThumbnail', $v));
						break;
					}
					
					$maxwidth = $v;
					break;

				case 'maxheight':
					
					if ($v <= 0)
					{
						trigger_error(sprintf('Invalid parameter maxheight=%s in OFThumbnail', $v));
						break;
					}
					
					$maxheight = $v;
					break;

				case 'innerborderwidth':
					$innerborderwidth = $v;
					break;
				case 'innerbordercolor':
					$innerbordercolor = $v;
					break;
				case 'innerborder':
					list($innerborderwidth, $innerbordercolor) = explode(',', $v);
					break;

				case 'outerborderwidth':
					$outerborderwidth = $v;
					break;
				case 'outerbordercolor':
					$outerbordercolor = $v;
					break;
				case 'outerborder':
					list($outerborderwidth, $outerbordercolor) = explode(',', $v);
					break;
					
				case 'resizemode':
					
					switch($v)
					{
						case 'KEEP_ASPECT_RATIO':
							$resizemode = Func_Thumbnailer::KEEP_ASPECT_RATIO;
							break;
						case 'CROP_CENTER':
							$resizemode = Func_Thumbnailer::CROP_CENTER;
							break;
					}
					
					break;
					
				case 'default':
					$default = $v;
					break;

				case 'saveas':
					$saveas = $v;
					break;
			}
		}
		
		
		
		// get image URL by method 
		$sourceset = false;

		foreach ($this->args as $p => $v) {
			switch (mb_strtolower(trim($p))) {
				
				case 'id_user':
					$sourceset = $this->setDirEntrySource($T, bab_getDirEntry($v, BAB_DIR_ENTRY_ID_USER));
					break;
		
				case 'id_directory_entry':
					$sourceset = $this->setDirEntrySource($T, bab_getDirEntry($v, BAB_DIR_ENTRY_ID));
					break;
					
				case 'id_article':
					$sourceset = $this->setArticleSource($T, $v);
					break;
					
				case 'path':
					$T->setSourceFile($v);
					$sourceset = true;
					break;
		
			}
		}
		
		if (!$sourceset && null !== $default)
		{
			
			$T->setSourceFile($default);
			$sourceset = true;
		}
	
		
		if ($sourceset)	
		{
			$T->setResizeMode($resizemode);
			$T->setBorder($outerborderwidth, $outerbordercolor, $innerborderwidth, $innerbordercolor);
			$imageUrl = $T->getThumbnail($maxwidth, $maxheight);
		} else {
			$imageUrl = '';
		}
		
		if (isset($saveas)) {
			$this->gctx->push($saveas, $imageUrl);
			return '';
		}
		
		return $imageUrl;
	}
	
	/**
	 * Set image source from directory entry array
	 * 
	 * @param Func_Thumbnailer $T
	 * @param Array $entry
	 * 
	 * return bool
	 */
	private function setDirEntrySource(Func_Thumbnailer $T, $entry)
	{
		if (!$entry)
		{
			bab_debug('OFThumbnail : directory entry does not exists');
			return false;
		}
		
		if (!isset($entry['jpegphoto']['photo']))
		{
			return false;
		}
		
		$photo = $entry['jpegphoto']['photo'];
		$data = $photo->getData();
		if (null !== $data && '' !== $data)
		{
			$T->setSourceBinary($data, $photo->lastUpdate());
			return true;
		}
		
		return false;
	}
	
	
	/**
	 * Set image source from article image
	 * 
	 * @param Func_Thumbnailer $T
	 * @param int $id_article
	 * @return boolean
	 */
	private function setArticleSource(Func_Thumbnailer $T, $id_article)
	{
		require_once $GLOBALS['babInstallPath'].'utilit/artapi.php';
		require_once $GLOBALS['babInstallPath'].'utilit/settings.class.php';
		
		$arr = bab_getImageArticle($id_article);
		
		if (!$arr)
		{
			// no photo
			return false;
		}
		
		$settings = bab_getInstance('bab_Settings');
		/*@var $settings bab_Settings */
		$site = $settings->getSiteSettings();
		
		$imagePath = $site['uploadpath'].'/'.$arr['relativePath'].$arr['name'];
		
		if (!file_exists($imagePath))
		{
			bab_debug(sprintf('Image not found for article %d with path : %s', $id_article, $imagePath));
			return false;
		}
		
		$T->setSourceFile($imagePath);
		return true;
	}
}
