<?php

require_once dirname(__FILE__).'/mock/mockObjects.php';

abstract class ORM_fieldTest extends PHPUnit_Framework_TestCase
{
    protected $fieldClass = 'ORM_Field';
    
    /**
     * @return ORM_Field
     */
    protected function construct()
    {
        require_once dirname(__FILE__) . '/../programs/field.class.php';
        $args = func_get_args();
        $field = $this->getMockForAbstractClass($this->fieldClass, $args);
        return $field;
    }


    protected function setUp()
    {

    }
    
    

    
    public function invalidNameProvider()
    {
        return array(
            array('1name'),
            array('a name'),
            array('a-name'),
//            array(''),
            array('abc/def')
        );
    }

    
    /**
     *
     */
    public function testNewField()
    {
        // Creates a Mock_ORM_Field.
        $field = $this->construct('name');
    
        $this->assertInstanceOf($this->fieldClass, $field);
    }
    
    
    
    
    /**
     * Trying to create a field with an invalid name must throw
     * an ORM_IllegalArgumentException.
     *
     * @dataProvider invalidNameProvider
     * @expectedException ORM_IllegalArgumentException
     */
    public function testCreateFieldWithInvalidName($name)
    {
        // Creates a Mock_ORM_Field.
        $field = $this->construct($name);
    }
    
    
    public function testPathIsSameAsNameForNewlyCreatedField()
    {
        // Creates a Mock_ORM_Field.
        $field = $this->construct('name');

        $this->assertEquals(
            $field->getName(),
            $field->getPath()
        );
    }
    
    

    public function testPathIsSameAsNameForFieldAddedToNamelessRecordSet()
    {
//        require_once dirname(__FILE__) . '/../programs/recordset.class.php';
        require_once dirname(__FILE__) . '/../programs/set.class.php';
        $set = $this->getMockForAbstractClass('ORM_RecordSet');
        
        // Creates a Mock_ORM_Field.
        $field = $this->construct('name');
        
        $set->addFields($field);
            
        $this->assertEquals(
            $field->getName(),
            $field->getPath()
        );
    }

    
    
    public function testPathOfFieldInNestedRecordSets()
    {
//        require_once dirname(__FILE__) . '/../programs/recordset.class.php';
        require_once dirname(__FILE__) . '/../programs/set.class.php';

        $set = new ORM_RecordSet();

        $subset = new ORM_TestRecordSet('subset');

        // Creates a Mock_ORM_Field.
        $field = $this->construct('name');

        $subset->addFields($field);

        $set->addFields($subset);

        $this->assertEquals(
            'subset/name',
            $field->getPath()
        );
    }
    

    
}
