<?php

require_once dirname(__FILE__).'/mock/mockObjects.php';
require_once dirname(__FILE__).'/fieldTest.php';

class ORM_fkFieldTest extends ORM_fieldTest
{
    protected $fieldClass = 'ORM_FkField';
    
    
    
    
    
    /**
     *
     */
    public function testNewField()
    {
        // Creates a Mock_ORM_Field.
        $field = $this->construct('name', 'testSet');
    
        $this->assertInstanceOf($this->fieldClass, $field);
    }
    
    /**
     * Trying to create a field with an invalid name must throw
     * an ORM_IllegalArgumentException.
     *
     * @dataProvider invalidNameProvider
     * @expectedException ORM_IllegalArgumentException
     */
    public function testCreateFieldWithInvalidName($name)
    {
        // Creates a Mock_ORM_Field.
        $field = $this->construct($name, 'testSet');
    }
    
    
    public function testPathIsSameAsNameForNewlyCreatedField()
    {
        // Creates a Mock_ORM_Field.
        $field = $this->construct('name', 'testSet');
    
        $this->assertEquals(
            $field->getName(),
            $field->getPath()
        );
    }
    
    
    public function testPathIsSameAsNameForFieldAddedToNamelessRecordSet()
    {
        require_once dirname(__FILE__) . '/../programs/set.class.php';
        $set = $this->getMockForAbstractClass('ORM_RecordSet');
    
        // Creates a Mock_ORM_Field.
        $field = $this->construct('name', 'testSet');
    
        $set->addFields($field);
    
        $this->assertEquals(
            $field->getName(),
            $field->getPath()
        );
    }
    
    
    public function testPathOfFieldInNestedRecordSets()
    {
        require_once dirname(__FILE__) . '/../programs/set.class.php';
    
        $set = new ORM_RecordSet();
    
        $subset = new ORM_TestRecordSet('subset');
    
        // Creates a Mock_ORM_Field.
        $field = $this->construct('name', 'testSet');
    
        $subset->addFields($field);
    
        $set->addFields($subset);
    
        $this->assertEquals(
            'subset/name',
            $field->getPath()
        );
    }
    
    
    /**
     * @expectedException ORM_Exception
     * @expectedExceptionMessage FkTestSet
     */
    public function testForeignSetWithoutClass()
    {
        $set = new ORM_MockSet();
        
        $field = $this->construct('fkname', 'FkTestSet');
        $set->addFields($field);
        
        $record = $set->newRecord();
        $record->fkname = 666;
        $record->fkname();
    }
    
    
    public function testForeignSetCreateSetNoRecord()
    {
        $set = new ORM_MockSet();
    
        $field = $this->construct('fkname', 'ORM_MockSet');
        $set->addFields($field);
    
        $record = $set->newRecord();
        $record->fkname = 666;
        $record = $record->fkname();
        
        $this->assertNull($record);
    }
    
    
    public function testForeignSetWithMockRecord()
    {
        $set = new ORM_MockSet();
        $foreignSet = new ORM_MockSet();
        $mockBackend = $set->getBackend();
    
        $field = $this->construct('fkname', 'ORM_MockSet');
        /*@var $field ORM_FkField */
        $field->setForeignSet($foreignSet);
        $set->addFields($field);
    
        $record = $set->newRecord();
        $record->fkname = '666';
        
        $foreignRecord = $foreignSet->newRecord();
        $foreignRecord->id = '666';
        
        $mockBackend->setSelectReturn($foreignSet, $foreignSet->id->is($record->fkname), array($foreignRecord));
        
        $record = $record->fkname();
    
        $this->assertInstanceOf('ORM_Mock', $record);
        $this->assertEquals('666', $record->id);
    }
    
    


    public function testJoiningForeignSetUsingJoinMethod()
    {
        $set = new ORM_MockSet();
        $foreignSetClassName = 'ORM_MockSet';
        $field1 = new ORM_FkField('fk', $foreignSetClassName);
        $set->addFields($field1);

        $fk = $set->join('fk');
        
        $this->assertInstanceOf('ORM_MockSet', $fk);
    }


    public function testJoiningForeignSetUsingMagicCallMethod()
    {
        $set = new ORM_MockSet();
        $foreignSetClassName = 'ORM_MockSet';
        $field1 = new ORM_FkField('fk', $foreignSetClassName);
        $set->addFields($field1);

        $set->fk();
        
        $this->assertInstanceOf('ORM_MockSet', $set->fk);
    }


    /**
     * After changing the foreign set on a fkField, joining this
     * fkField using the join() method MUST return the same record set.
     */
    public function testChangingForeignSetHasEffectOnJoinMethod()
    {
        $set = new ORM_MockSet();
        $set->hasOne('fk', 'ORM_MockSet');
    
        $newRecordSet = new ORM_TestRecordSet();
        $set->fk->setForeignSet($newRecordSet);
        
        $set->join('fk');
        
        $this->assertSame(
            $set->fk,
            $newRecordSet,
            'After changing the foreign set on a fkField using setForeignSet, joining this fkField using the join() method MUST return the same record set'
        );
    }

    
    /**
     * After changing the foreign set on a fkField, joining this
     * fkField using the join() method MUST return the same record set.
     */
    public function testChangingForeignSetHasEffectOnMagicCallMethod()
    {
        $set = new ORM_MockSet();
        $set->hasOne('fk', 'ORM_MockSet');
    
        $newRecordSet = new ORM_TestRecordSet();
        $set->fk->setForeignSet($newRecordSet);
    
        $set->fk();
    
        $this->assertSame(
            $set->fk,
            $newRecordSet,
            'After changing the foreign set on a fkField using setForeignSet, joining this fkField using the __call() method MUST return the same record set'
        );
    }
}
