<?php
//-------------------------------------------------------------------------
// OVIDENTIA http://www.ovidentia.org
// Ovidentia is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
// USA.
//-------------------------------------------------------------------------
/**
 * @license http://opensource.org/licenses/gpl-license.php GNU General Public License (GPL)
 * @copyright Copyright (c) 2006 by CANTICO ({@link http://www.cantico.fr})
 */
require_once 'base.php';

require_once $GLOBALS['babInstallPath'].'utilit/artapi.php';
require_once $GLOBALS['babInstallPath'].'utilit/urlincl.php';
require_once $GLOBALS['babInstallPath'].'utilit/json.php';
require_once dirname(__FILE__).'/functions.php';


function editlinks_topicssub($id_topic)
{
    return bab_isAccessValid('bab_topicssub_groups', $id_topic);
}

function editlinks_topicsman($id_topic)
{
    return bab_isAccessValid('bab_topicsman_groups', $id_topic);
}


/**
 * Update json object with publication items
 * get allowed pubnodes to publish into
 *
 *
 */
function editlinks_addPublicationItems($json)
{
    $publication = bab_functionality::get('Publication');


    if (false === $publication) {
        return;
    }



    if ($pubnodes = (array) bab_gp('pubnodes', array())) {

        $json->label->pubnodes = array();

        /*@var $publication Func_Publication */
        $pubNodeSet = $publication->pubNodeSet();

        $json->pubnodes = array();
        $res = $pubNodeSet->select($pubNodeSet->id->in($pubnodes));
        foreach($res as $pubNode) {
            if ($pubNode->canCreateDynamicRecord()) {
                $json->pubnodes[] = (int) $pubNode->id;

                try {
                    $structure = $pubNode->getStructure();

                } catch(Exception $e) {
                    bab_debug($e->getMessage());
                    $description = '???';
                }

                $create = $structure->getCreateTitle();
                if (!isset($create)) {
                    $create = editlinks_translate('Create');
                }

                $json->label->pubnodes[$pubNode->id] = $create;
            }
        }

    }


    if ($dynamicrecords = (array) bab_gp('dynamicrecords', array())) {
        $json->dynamicrecords = array();
        foreach ($dynamicrecords as $structure => $arr) {
            $set = $publication->xsd()->getStructureByName($structure)->getRecordSet();

            try {
                $res = $set->select($set->id->in($arr));
                foreach($res as $record) {
                    /* @var $record \Ovidentia\Publication\DynamicRecord */
                    if ($record->canUpdate()) {

                        if (!isset($json->dynamicrecords[$structure])) {
                            $json->dynamicrecords[$structure] = array();
                        }
                        
                        $nodes = $record->getCustomNodes();
                        $sitemapItems = array();
                        
                        foreach($nodes as $node) {
                            $item = $node->getData();
                            /*@var $item bab_siteMapItem */
                            $sitemapItems[] = array(
                                'id'            => $item->id_function,
                                'langId'        => $item->langId,
                                'name'          => $item->name,
                                'description'   => $item->description,
                                'iconClassnames'=> $item->iconClassnames
                            );
                        }

                        $json->dynamicrecords[$structure][] = array(
                            'id' => (int) $record->id,
                            'nodes' => $sitemapItems
                        );
                    }
                }

            } catch (ORM_BackEndSelectException $e) {
                // the table does not exists or is not in sync with structure
                bab_debug($e->getMessage());
                continue;
            }
        }
    }
    
    
    if ($nodedynamicrecords = (array) bab_gp('nodedynamicrecords', array())) {
        
        $json->nodedynamicrecords = array();
        
        if (bab_isUserAdministrator()) {
            
            foreach ($nodedynamicrecords as $structure => $sitemapNodes) {
    
                foreach($sitemapNodes as $sitemapNode) {
    
                    if (!isset($json->nodedynamicrecords[$structure])) {
                        $json->nodedynamicrecords[$structure] = array();
                    }
    
                    $json->nodedynamicrecords[$structure][] = $sitemapNode;
                
                }
            }
        }
    }



    $json->label->dynamicrecords = $dynamicrecord = new stdClass();
    $json->label->nodedynamicrecords = $nodedynamicrecords = new stdClass();

    foreach($publication->xsd()->getStructures() as $structure) {
        $name = $structure->getName();
        
        $create = $structure->getCreateTitle();
        if (!isset($create)) {
            $create = editlinks_translate('Create');
        }
        
        $edit = $structure->getEditTitle();
        if (!isset($edit)) {
            $edit = editlinks_translate('Edit');
        }

        $dynamicrecord->$name  = $edit;
        $nodedynamicrecords->$name = $create;
    }

}


/**
 * Link to node display in sitemap editor
 * @param object $json
 */
function editlinks_addSitemapItems($json)
{
    $json->sitemapnodes = array();
    
    if (!bab_isUserAdministrator()) {
        return;
    }
    
    if ($sitemapnodes = (array) bab_gp('sitemapnodes', array())) {
        $siteSitemap = bab_siteMap::getFromSite();
        $siteRootNode = bab_siteMap::getVisibleRootNodeSitemapItem();

        foreach($sitemapnodes as $nodeId) {
            $node = $siteSitemap->getNodeById($nodeId);
            
            // ensure the node is under Custom
            
            do {
                if ($node->getId() === $siteRootNode->id_function) {
                    $json->sitemapnodes[] = $nodeId;
                }
            } while (isset($node) && $node = $node->parentNode());
        }
        
        
        $json->label->sitemapnodes = editlinks_translate('View sitemap node');
    }
}



function editlinks_filterInt($paramName, $funcName)
{
    return array_map(
        create_function('$value', 'return (int) $value;'),
        array_filter((array) bab_gp($paramName), $funcName)
    );
}


function editlinks_getJSON()
{


    $url = new bab_url(bab_gp('url'));
    $registry = bab_getRegistryInstance();
    $registry->changeDirectory('/editlinks/');


    // Disable editlinks on articles pages

    if ($url->tg === 'articles' && is_numeric($url->topics) && true !== $registry->getValue('topic'))
    {
        die();
    }

    if ($url->tg === 'articles' && $url->idx === 'More' && is_numeric($url->article) && true !== $registry->getValue('readmore'))
    {
        die();
    }



    // build JSON object

    $json = new stdClass();
    $json->articles = editlinks_filterInt('articles', 'bab_isArticleModifiable');
    $json->topicssub = editlinks_filterInt('topics', 'editlinks_topicssub');
    $json->topicsman = editlinks_filterInt('topics', 'editlinks_topicsman');

    $json->label = new stdClass();

    $json->label->articles = editlinks_translate('Edit article');
    $json->label->topicssub = editlinks_translate('Add article');
    $json->label->topicsman = editlinks_translate('Manage topic');

    editlinks_addPublicationItems($json);
    editlinks_addSitemapItems($json);

    return bab_convertStringFromDatabase(bab_json_encode($json), 'UTF-8');

}

die(editlinks_getJSON());